@@ -71,6 +71,9 @@ class Node:
             "merge_cubes": MergeCubesNode,
             "if": IfNode,
             "apply": ApplyNode,
+            "count": CountNode,
+            "array_apply": ArrayApplyNode,
+            "array_filter": ArrayFilterNode,
         }
         if class_types_for_process.get(process_id):
             self.__class__ = class_types_for_process[process_id]
@@ -245,3 +248,58 @@ function apply(arguments) {{
     return newData;
 }}
 """
+
+
+class CountNode(Node):
+    def write_process(self):
+        newline = "\n"
+        tab = "\t"
+        return f"""
+function count(arguments) {{
+    function condition(arguments) {{
+    {newline.join(node.write_function() for node in self.child_nodes)}
+    {newline.join(node.write_call() for node in self.child_nodes)}
+        return {(self.child_nodes[-1].node_varname_prefix + self.child_nodes[-1].node_id) if len(self.child_nodes) > 0 else None};
+    }}
+
+    {pkgutil.get_data("pg_to_evalscript", f"{self.process_definitions_directory}/is_valid.js").decode("utf-8")}
+
+    {self.load_process_code()}
+    return count(arguments);
+}}
+"""
+
+class ArrayApplyNode(Node):
+    def write_process(self):   
+        newline = "\n"
+        tab = "\t"
+        return f"""
+function array_apply(arguments) {{
+    function process(arguments) {{
+    {newline.join(node.write_function() for node in self.child_nodes)}
+    {newline.join(node.write_call() for node in self.child_nodes)}
+        return {self.child_nodes[-1].node_varname_prefix + self.child_nodes[-1].node_id};
+    }}
+
+    {self.load_process_code()}
+    const {{ data, context }} = arguments;
+    return array_apply({{ data: data, context: context, process: process }})
+}}
+"""
+
+class ArrayFilterNode(Node):
+    def write_process(self):
+        newline = "\n"
+        tab = "\t"
+        return f"""
+function array_filter(arguments) {{
+    function condition(arguments) {{
+    {newline.join(node.write_function() for node in self.child_nodes)}
+    {newline.join(node.write_call() for node in self.child_nodes)}
+        return {self.child_nodes[-1].node_varname_prefix + self.child_nodes[-1].node_id};
+    }}
+
+    {self.load_process_code()}
+    return array_filter(arguments);
+}}
+"""
